<?php

namespace Slack\Options;

use Illuminate\Support\Facades\Log;

use Slack\Base as SlackBase;
use Slack\Message;

abstract class Base extends SlackBase
{
	private const LOGKEY = 'SOb';

	// Does the event respond with a reply to the HTTP request, or via a post with a trigger
	// Child class should have a respond() function.
	// (There should be a local implementation of the child class should respondNow = TRUE)
	public $respondNow = FALSE;

	public function __construct(array $request)
	{
		Log::info(sprintf('%s:Slack INTERACTIVE MESSAGE Initialised [%s]',self::LOGKEY,get_class($this)),['m'=>__METHOD__]);

		// Our data is in a payload value
		parent::__construct($request);
	}

	/**
	 * Enable getting values for keys in the response
	 *
	 * @note: This method is limited to certain values to ensure integrity reasons
	 * @note: Classes should return:
	 *    + channel_id,
	 *    + team_id,
	 *    + ts,
	 *    + user_id
	 * @param string $key
	 * @return mixed|object
	 */
	public function __get(string $key)
	{
		switch ($key) {
			case 'team_id':
				return object_get($this->_data,'team.id');
			case 'channel_id':
				return object_get($this->_data,'channel.id');

			case 'enterprise_id':
				return object_get($this->_data,'team.'.$key);

			case 'user_id':
				return object_get($this->_data,'user.id');

			case 'callback_id':
			//case 'action_ts':
			//case 'message_ts':
			case 'type':
				return object_get($this->_data,$key);
		}
	}

	/**
	 * Interactive messages can return their output in the incoming HTTP post
	 *
	 * @return Message
	 * @throws \Exception
	 */
	public function respond(): Message
	{
		Log::info(sprintf('%s:Interactive Option - Callback [%s] Name [%s] Value [%s]',self::LOGKEY,$this->callback_id,$this->name,$this->value),['m'=>__METHOD__]);

		if (preg_match('/^(.*)\|([0-9]+)/',$this->callback_id)) {
			[$action,$id] = explode('|',$this->callback_id,2);

		} else {
			// If we get here, its an action that we dont know about.
			Log::notice(sprintf('%s:Unhandled CALLBACK [%s]',self::LOGKEY,$this->callback_id),['m'=>__METHOD__]);
		}

		return Message::blank();
	}
}